/**
 * @file Click_Pollution_WSP2110.c
 * @brief <h2> Pollution click Example </h2>
 *
 * @par
 * Example for <a href="http://www.mikroe.com">MikroElektronika's</a>
 * Pollution click board.
 *
 *
 *
 ******************************************************************************/

/**
 * @page        Example
 * @date        15 Nov 2016
 * @author      Branislav Marton
 * @copyright   GNU Public License
 * @version     1.0.0 - Initial testing and verification
 */

/**
 * @page TEST_CFG Test Configurations
 *
 * ### Test configuration STM : ###
 * @par
 * -<b> MCU           </b> :    STM32F207VG
 * -<b> Dev. Board    </b> :    EasyMx v7 for STM32
 * -<b> Ext. Modules  </b> :    Pollution click board, EasyTFT
 * -<b> SW            </b> :    mikroC PRO for ARM v4.9.0
 *
 */

 /**
  * @page Dev. board setup
  *
  * 1) Put Pollution click in mikroBUS 1 slot.
  * 2) Button press level for PORTB/H set to GND.
  * 3) Set pull-up for PB8.
  */

/******************************************************************************
* Includes
*******************************************************************************/
#include "pollution.h"

/******************************************************************************
* Preprocessors
*******************************************************************************/
#define C1  0.8639656653028277      // Constant for calculating ppm (I segment)
#define C2  1.1496082990709344      // Constant for calculating ppm (II segment)
#define C3  1.5489389323538676      // Constant for calculating ppm (III segment)

#define M1  -0.30361905759780833    // Constant for calculating ppm (I segment)
#define M2  -0.4694852833012203     // Constant for calculating ppm (II segment)
#define M3  -0.5989696888059545     // Constant for calculating ppm (III segment)

#define V_REF 3.3
#define ADC_STEP V_REF / 1023

#define VCC 4.95

#define CHANNEL 8

/******************************************************************************
* Typedefs
*******************************************************************************/

/******************************************************************************
* Variable Definitions
*******************************************************************************/


/******************************************************************************
* Function Prototypes
*******************************************************************************/

/******************************************************************************
* Private Functions
*******************************************************************************/

/******************************************************************************
* Public Functions
*******************************************************************************/

/***************************************************************************//**
 *  Measure load voltage form ADC channel.
 *  \return Load voltage.
 */
float measure_v_load()
{
    uint16_t adc_value;

    adc_value = ADC1_Read(CHANNEL) & 0x0FFF;
    return adc_value * ADC_STEP;
}

/***************************************************************************//**
 *  Calculate sensor resistance for given load voltage and known 
 *  load resistance.
 *  \param v_load Measured load voltage.
 *  \param r_load Known load resistance.
 *  \return Sensor resistance.
 */
float sensor_resistance(float v_load, float r_load)
{
    return r_load * (VCC / v_load - 1);
}

/***************************************************************************//**
 *  Caclulate load resistance for given load voltage and known sensor
 *  resistance.
 */
float load_resistance(float v_load, float r_sensor)
{
    return r_sensor * (v_load / (VCC - v_load) );
}

/***************************************************************************//**
 *  Function for calculating ppm for given ratio Rs/Ro. 
 *  \param ratio Input ratio Rs/Ro.
 *  \return Returns calculated ppm.
 */
float ratio_to_ppm(float ratio)
{
    if (0.84 > ratio && ratio >= 0.53)      // I segment
        return pow((ratio/C1), (1/M1));
    
    else if (0.53 > ratio && ratio >= 0.39) // II segment
        return pow((ratio/C2), (1/M2));

    else if (0.39 > ratio && ratio >= 0.148)  // III segment
        return pow((ratio/C3), (1/M3));

    else if (ratio < 0.148)
        return 50.0;

    return 1.0;    
}

/*************** END OF FUNCTIONS *********************************************/
