/**
 * @file Click_Pollution_WSP2110.c
 * @brief <h2> Pollution click Example </h2>
 *
 * @par
 * Example for <a href="http://www.mikroe.com">MikroElektronika's</a>
 * Pollution click board.
 *
 *
 *
 ******************************************************************************/

/**
 * @page        Example
 * @date        15 Nov 2016
 * @author      Branislav Marton
 * @copyright   GNU Public License
 * @version     1.0.0 - Initial testing and verification
 */

/**
 * @page TEST_CFG Test Configurations
 *
 * ### Test configuration STM : ###
 * @par
 * -<b> MCU           </b> :    STM32F207VG
 * -<b> Dev. Board    </b> :    EasyMx v7 for STM32
 * -<b> Ext. Modules  </b> :    Pollution click board, EasyTFT
 * -<b> SW            </b> :    mikroC PRO for ARM v4.9.0
 *
 */

 /**
  * @page Dev. board setup
  *
  * 1) Put Pollution click in mikroBUS 1 slot.
  * 2) Button press level for PORTB/H set to GND.
  * 3) Set pull-up for PB8.
  */

/******************************************************************************
* Includes
*******************************************************************************/
#include "pollution_gui.h"

#include "tft_resources.h"

/******************************************************************************
* Preprocessors
*******************************************************************************/


/******************************************************************************
* Typedefs
*******************************************************************************/

/******************************************************************************
* Variable Definitions
*******************************************************************************/
static char str_buffer[16];

static char inequality_sign = ' ';

char TFT_DataPort                   at LATE;
sbit TFT_RST                        at LATD7_bit;
sbit TFT_BLED                       at LATD2_bit;
sbit TFT_RS                         at LATD9_bit;
sbit TFT_CS                         at LATD10_bit;
sbit TFT_RD                         at LATD5_bit;
sbit TFT_WR                         at LATD4_bit;
char TFT_DataPort_Direction         at TRISE;
sbit TFT_RST_Direction              at TRISD7_bit;
sbit TFT_BLED_Direction             at TRISD2_bit;
sbit TFT_RS_Direction               at TRISD9_bit;
sbit TFT_CS_Direction               at TRISD10_bit;
sbit TFT_RD_Direction               at TRISD5_bit;
sbit TFT_WR_Direction               at TRISD4_bit;

/******************************************************************************
* Function Prototypes
*******************************************************************************/

/******************************************************************************
* Private Functions
*******************************************************************************/

/******************************************************************************
* Public Functions
*******************************************************************************/

void gui_init()
{
    display_init("Pollution click", "WSP2110", "EasyPIC Fusion v7");
}

/***************************************************************************//**
 *  Prints 'calibration' page with isntructions.
 */
void gui_print_calibration_page()
{
    TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL );
    TFT_Write_Text( "Calibration: V(load) = ", 20, 80 );

    TFT_Set_Font( &Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL );
    TFT_Write_Text("INSTRUCTION: Put sensor in clean air and adjust ", 20, 150);
    TFT_Write_Text("potentiometer on the click board farthest to", 20, 165 );
    TFT_Write_Text("the right. Wait for sensor to stabilise and than", 20, 180);
    TFT_Write_Text("press PB8.", 20, 195 );
}

/***************************************************************************//**
 *  Prints 'Adjust sensitivity' page with isntructions.
 */
void gui_print_adjust_load_page()
{
    TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL );
    TFT_Write_Text( "R(load) = ", 20, 80 );

    TFT_Set_Font( &Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL );
    TFT_Write_Text( "INSTRUCTION: Adjust potentimeter to prefered  ", 20, 150 );
    TFT_Write_Text( "value, to define sensitivity. When done press", 20, 165 );
    TFT_Write_Text( "PB8.", 20, 180 );
}

/***************************************************************************//**
 *   Prints 'Measure' page.
 */
void gui_print_measure_page()
{ 
    TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL );
    TFT_Write_Text( "Rs/Ro", 120, 100 );
    TFT_Write_Text( "ppm", 120, 160 );

    TFT_Image(20, 100, &biohazard_bmp, 1);
    
    TFT_Set_Pen( CL_Black, 1 );
    TFT_Set_Brush( 1, CL_WHITE, 0, 0, 0, 0 );
    
    TFT_Rectangle(200, 90, 300, 130);
    TFT_Rectangle(200, 150, 300, 190);
}

/***************************************************************************//**
 *   Clears page area.
 */
void gui_clear_page()
{
    TFT_Set_Pen( CL_WHITE, 0 );
    TFT_Set_Brush( 1, CL_WHITE, 0, 0, 0, 0 );
    
    TFT_Rectangle( 20, 71, 300,  219);
}

/***************************************************************************//**
 *  Refresh value on page.
 *  \param value New value to display.
 *  \param old_value Latest displayed value.
 *  \param x Text X position.
 *  \param y Text Y position.
 *  \param format An sprintf format.
 *  \return Return value.
 */
float gui_refresh_value(
    float value, 
    float old_value, 
    uint16_t x, 
    uint16_t y,
    const char* format
)
{
    if(value != old_value)
    {
        // Erase old value.
        TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_WHITE, FO_HORIZONTAL );
        sprintf(str_buffer, format, old_value);
        TFT_Write_Text( str_buffer, x, y );
        // Print new value.
        TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL );
        sprintf(str_buffer, format, value);
        TFT_Write_Text( str_buffer, x, y );
    }

    return value;
}

/***************************************************************************//**
 *  Updates inequality sign on display.
 *  \param value PPM value to compare.
 */
void gui_update_inequality_sign(float value)
{
    char new_inequality_sign;
    new_inequality_sign = (value == 50) ? '>' : ((value == 1) ? '<' : ' ');

    if(new_inequality_sign != inequality_sign)
    {
        TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_WHITE, FO_HORIZONTAL );
        TFT_Write_Char(inequality_sign, 210, 160);

        TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL );
        TFT_Write_Char(new_inequality_sign, 210, 160);

        inequality_sign = new_inequality_sign;
    }    
}

/*************** END OF FUNCTIONS *********************************************/